from datetime import datetime
from unittest.mock import patch
from raw_image_converter.utils import message


class TestMessage:
    # Case: converted
    def test_print_converted_file_message(self):
        file = "example.jpg"
        converted = True
        expected_message = (
            f"{datetime.now().time().strftime('%H:%M:%S')} Converted: {file}"
        )

        with patch("builtins.print") as mock_print:
            message(file, converted)

        mock_print.assert_called_once_with(expected_message)

    # Case: NOT converted
    def test_print_failed_conversion_message(self):
        file = "example.jpg"
        converted = False
        expected_message = f"{datetime.now().time().strftime('%H:%M:%S')} Conversion failed for File: {file}"

        with patch("builtins.print") as mock_print:
            message(file, converted)

        mock_print.assert_called_once_with(expected_message)



from datetime import datetime
import os
from raw_image_converter import __version__
from raw_image_converter.utils import (
    check_file_type,
    convert_file,
    convert_raw,
    image_not_exists,
    delete_directory,
)
import argparse
import concurrent.futures
from colorama import Fore, Style


def tuple_type(values):
    values = values.split(",")
    return tuple(values)


def main():
    print(f"### PYTHON IMAGE CONVERTER {__version__} ### \n \n")

    parser = argparse.ArgumentParser(description="Convert images to JPG/PNG")
    parser.add_argument(
        "-s",
        "--src",
        dest="src_dir",
        help="specify the source directory!",
        required=True,
    )
    parser.add_argument(
        "-t",
        "--tgt",
        dest="tgt_dir",
        help="specify the target directory!",
        default="converted",
    )
    parser.add_argument(
        "-e",
        "--ext",
        dest="ext",
        default=".jpg",
        choices=[".jpg", ".png"],
        help="the image format to be used for the converted images.",
    )
    parser.add_argument(
        "-d",
        "--delete-source-directory",
        action="store_true",
        dest="delete_source_directory",
        default=False,
        help="Delete the source directory after the convesion",
    )
    parser.add_argument(
        "-r",
        "--resolution",
        type=tuple_type,
        dest="resolution",
        default="100%,100%",
        help="image dimensions (width, height) as a tuple (using numbers 800,600 or using percentages 75%,75%)",
    )

    args = parser.parse_args()

    srcDir = os.path.abspath(args.src_dir)
    tgtDir = os.path.abspath(args.tgt_dir)
    resolution = args.resolution

    # Handle one dimensional tuples
    if len(resolution) == 1:
        resolution = (resolution[0], resolution[0])

    if not os.path.exists(tgtDir):
        os.makedirs(tgtDir)

    # find files to convert
    try:
        with concurrent.futures.ProcessPoolExecutor() as executor:
            print(
                "Started conversion at : "
                + datetime.now().time().strftime("%H:%M:%S")
                + "\n"
            )
            print("Converting -> " + srcDir + " Directory !\n")
            for file in os.listdir(srcDir):
                file = file.lower()
                if image_not_exists(file, tgtDir, args.ext):
                    type = check_file_type(file)
                    if "RAW" == type:
                        executor.submit(
                            convert_raw,
                            file,
                            srcDir,
                            tgtDir,
                            args.ext,
                            resolution,
                        )

                    if "NOT_RAW" == type:
                        executor.submit(
                            convert_file,
                            file,
                            srcDir,
                            tgtDir,
                            args.ext,
                            resolution,
                        )
                else:
                    print(
                        f"{Fore.GREEN}File "
                        + file
                        + f" is already converted!{Style.RESET_ALL}"
                        + " \n "
                    )

        print(
            f"{Fore.GREEN}Converted Images are stored at - > "
            + os.path.abspath(tgtDir)
            + f"{Style.RESET_ALL}"
        )

        if args.delete_source_directory:
            delete_directory(srcDir)

    except Exception as e:
        print(f"{Fore.RED}ERROR IN APPLICATION{Style.RESET_ALL}" + e)


if __name__ == "__main__":
    main()



from unittest.mock import patch
from raw_image_converter.utils import check_file_type


class TestCheckFileType:
    # Case: Returns "RAW" if file extension is raw
    def test_returns_raw_if_extension_in_raw_conversion_array(self):
        extensions= [
            ".dng",
            ".raw",
            ".cr2",
            ".crw",
            ".erf",
            ".raf",
            ".kdc",
            ".dcr",
            ".mos",
            ".mef",
            ".nef",
            ".orf",
            ".rw2",
            ".pef",
            ".x3f",
            ".srw",
            ".srf",
            ".sr2",
            ".arw",
            ".mdc",
            ".mrw",
        ]

        for ext in extensions:
            assert check_file_type(f"file{ext}") == "RAW"

    # Case: Returns "NOT RAW" if file extension is not raw
    def test_returns_not_raw_if_extension_in_conversion_array(self):
        extensions2 = [".ppm", ".psd", ".webp", ".tiff"]

        for ext in extensions2:
            assert check_file_type(f"file{ext}") == "NOT RAW"

    # Case: Calls ai_2_pdf function if file extension is ".ai"
    def test_calls_ai_2_pdf_if_extension_is_ai(self):
        with patch("os.rename") as mock_rename:
            check_file_type("file.ai")
            mock_rename.assert_called_once_with("file.ai", "file.ai.pdf")




from raw_image_converter.utils import calculate_image_dimension


class TestCalculateImageDimension:
    # Case: resolution is a set number
    def test_valid_integer_dimension_and_resolution(self):
        dimension = 100
        resolution = "200"
        expected_result = 200

        result = calculate_image_dimension(dimension, resolution)

        assert result == expected_result

    # Case: resolution is a percentage
    def test_valid_integer_dimension_and_percentage_resolution(self):
        dimension = 100
        resolution = "50%"
        expected_result = 50

        result = calculate_image_dimension(dimension, resolution)

        assert result == expected_result



from PIL import Image
import os
import numpy as np
import rawpy
import imageio
from datetime import datetime
import shutil

def split_file_extension(file):
    return os.path.splitext(file)

def message(file, converted):
    current_time = datetime.now().time().strftime("%H:%M:%S")
    # if conversion finished
    if converted:
        print(f"{current_time} Converted: {file}")
    # if conversion failed
    else:
        print(f"{current_time} Conversion failed for File: {file}")


# convert RAW images function
def convert_raw(file, srcDir, tgtDir, extension=".jpg", resolution=("100%", "100%")):
    try:
        filename, _ = split_file_extension(file)
        print(datetime.now().time().strftime("%H:%M:%S") + " Converting:  " + file)
        source = os.path.join(srcDir, file)
        with rawpy.imread(source) as raw:
            rgb = raw.postprocess()
        if resolution:
            pil_image = Image.fromarray(rgb)

            # Resize the image
            width = calculate_image_dimension(pil_image.width, resolution[0])
            height = calculate_image_dimension(pil_image.height, resolution[1])
            pil_image = pil_image.resize((width, height))

            rgb = np.array(pil_image)
        imageio.imsave(os.path.join(tgtDir, filename + extension), rgb)
        message(file, True)
    except Exception as e:
        print(e)
        message(file, False)
        pass


def calculate_image_dimension(dimension, resolution):
    if "%" in resolution:
        factor = float(resolution.strip("%").strip()) / 100.0
        result = int(dimension * factor)
    else:
        result = int(resolution)
    return result


# convert function
def convert_file(file, srcDir, tgtDir, extension=".jpg", resolution=("100%", "100%")):
    mappings = {
        ".jpg": "JPEG",
        ".png": "PNG",
    }
    save_format = mappings.get(extension, "JPEG")
    try:
        filename, _ = split_file_extension(file)
        message(file, False)
        path = os.path.join(srcDir, file)
        im = Image.open(path)
        if resolution:
            width = calculate_image_dimension(im.width, resolution[0])
            height = calculate_image_dimension(im.height, resolution[1])
            im = im.resize((width, height))
        im.save(os.path.join(tgtDir, filename + extension), save_format)
        message(file, True)
    except:
        pass


# rename .ai 2 pdf and problem solved!
def ai_2_pdf(file):
    if file.endswith(".ai"):
        os.rename(file, os.path.join(file + ".pdf"))
        print(
            datetime.now().time().strftime("%H:%M:%S")
            + " Converted ai 2 pdf : "
            + os.path.join(file + ".pdf")
        )


# IT IS POINTLESS TO CONVERT WHAT IS ALREADY CONVERTED!!!!
def image_not_exists(image, tgtDir, tgtExt):
    filename, _ = split_file_extension(image)
    target = os.path.join(tgtDir, filename + tgtExt)
    if os.path.isfile(target):
        return False
    else:
        return True


# here we check each file to decide what to do
def check_file_type(file):
    # get the extension as a String and check if the string is contained in the array extensionsForRawConversion
    _, ext = split_file_extension(file)
    # set supported raw conversion extensions!
    raw = [
        ".dng",
        ".raw",
        ".cr2",
        ".crw",
        ".erf",
        ".raf",
        ".kdc",
        ".dcr",
        ".mos",
        ".mef",
        ".nef",
        ".orf",
        ".rw2",
        ".pef",
        ".x3f",
        ".srw",
        ".srf",
        ".sr2",
        ".arw",
        ".mdc",
        ".mrw",
    ]
    # set supported imageio conversion extensions
    not_raw = [".ppm", ".psd", ".tiff", ".webp"]

    if ext in raw:
        return "RAW"
    if ext in not_raw:
        return "NOT RAW"
    # check if an .ai exists and rename it to .pdf	!
    ai_2_pdf(file)


def delete_directory(dir):
    try:
        shutil.rmtree(dir)
        print(f"Removed source directory {dir}")
    except OSError as o:
        print(f"Error, {o.strerror}: {dir}")



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="raw-image-converter",
    version="1.1.1",
    description="Batch conversions of raw images",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/achimoraites/Python-Image-Converter",
    author="Achilles Moraites",
    author_email="achimoraites@yahoo.gr",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        'Topic :: Utilities',
    ],
    keywords='cli, converter, raw, images',
    packages=["raw_image_converter"],
    install_requires=["numpy==1.22.3", "rawpy==0.17.1",
                      "imageio==2.16.2", "Pillow==10.0.1", "colorama==0.4.6"],
    entry_points={
        "console_scripts": [
            "raw_image_converter=raw_image_converter.__main__:main",
        ]
    },
)



from raw_image_converter.utils import split_file_extension


class TestSplitFileExtension:
    # Case: Test with a file name with an extension
    def test_extension(self):
        file = "example.jpg"
        result = split_file_extension(file)
        assert result == ("example", ".jpg")

    # Case: Test with a file name with no extension
    def test_no_extension(self):
        file = "example"
        result = split_file_extension(file)
        assert result == ("example", "")

    # Case: Test with an empty file name
    def test_empty_file_name(self):
        file = ""
        result = split_file_extension(file)
        assert result == ("", "")

    # Case: Test with a file name starting with a dot
    def test_starting_with_dot(self):
        file = ".example"
        result = split_file_extension(file)
        assert result == (".example", "")

    # Case: Test with a file name containing multiple dots
    def test_multiple_dots(self):
        file = "example.file.with.dots"
        result = split_file_extension(file)
        assert result == ("example.file.with", ".dots")
